/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension.data;

import java.io.IOException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.IStorageProvider;

/**
 * Interface for accessing top-level elements provided by
 * {@link IStorageProvider}s. This interface supports the typical methods like
 * running commands, listening to the command stack, undo and redo, a dirty
 * state and a save mechanism as well as accessing the root model element.
 * 
 * <P>
 * Once an instance of this interface is provided to the kernel, the underlying
 * load must be completed successfully. Failure during the model loading must be
 * handled by the storage provider. The kernel must only become aware of loaded
 * models.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 627F8F0B59A57FB8FE9CD20F70C02C7F
 */
public interface ITopLevelElement {

	/** Returns the root model element represented by this top-level element. */
	// TODO: vielleicht wre ne Definition, was ein Top-Level-Element und was
	// ein Root-Element ist, nett. Es wre auf jeden Fall hilfreich, da das
	// keine 100% selbsterklrenden Begriffe sind.
	EObject getRootModelElement();

	/** Executes the given {@link Runnable} as model changing command. */
	// TODO: warum model changing? Was fr ein Model wird wie gendert und warum
	// kommt das im Methodennamen nicht vor?
	void runAsCommand(Runnable runner);

	/**
	 * Adds a command stack listener. Has no effect if an identical listener is
	 * already registered.
	 */
	void addCommandStackListener(CommandStackListener listener);

	/** Removes the command stack listener. */
	void removeCommandStackListener(CommandStackListener listener);

	/** Returns whether undo operation is possible. */
	boolean canUndo();

	/** Returns whether redo operation is possible. */
	boolean canRedo();

	/** Executes the undo operation. */
	void undo();

	/** Executes the redo operation. */
	void redo();

	/** Returns whether there is unsaved content. */
	boolean isDirty();

	/** Returns the name of the saveable unit. */
	String getSaveableName();

	/** Performs the save operation. */
	void doSave(IProgressMonitor monitor) throws CoreException, IOException;

	/**
	 * Prepares the IDs of the given element and its children before the element
	 * becomes part of this top-level elements model element tree.
	 */
	void prepareIDs(EObject modelElement);

	/** Returns whether this top-level element can be deleted. */
	boolean canDelete();

	/**
	 * Deletes this top-level element from the underlying persistency solution.
	 * Returns <code>false</code> if the delete was interrupted by, e.g., the
	 * user.
	 */
	boolean delete();
}
